package manalichricard.android.uoc.edu.discaparking;

import android.content.Intent;
import android.graphics.Bitmap;
import android.provider.MediaStore;
import android.support.annotation.NonNull;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.view.View;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.RadioButton;
import android.widget.Toast;

import com.google.android.gms.tasks.OnFailureListener;
import com.google.android.gms.tasks.OnSuccessListener;
import com.google.firebase.storage.FirebaseStorage;
import com.google.firebase.storage.StorageReference;
import com.google.firebase.storage.UploadTask;

import java.io.ByteArrayOutputStream;
import manalichricard.android.uoc.edu.discaparking.model.Parking;

public class AddParkingActivity extends AppCompatActivity {

    private Parking mParking;
    private EditText editText_places;
    private CheckBox checkBox_left_load, checkBox_right_load, checkBox_rear_load;
    private RadioButton radioButton_batery, radioButton_line;
    private String id;
    private String address;
    private Double latitude;
    private Double longitude;
    private static final int REQUEST_IMAGE_CAPTURE = 1;
    private FirebaseStorage firebaseStorage;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_add_parking);

        radioButton_batery = (RadioButton) findViewById(R.id.rb_add_parking_batery);
        radioButton_line = (RadioButton) findViewById(R.id.rb_add_parking_line);
        checkBox_left_load = (CheckBox)findViewById(R.id.cb_add_parking_left_load);
        checkBox_right_load = (CheckBox)findViewById(R.id.cb_add_parking_right_load);
        checkBox_rear_load = (CheckBox)findViewById(R.id.cb_add_parking_rear_load);
        editText_places = (EditText)findViewById(R.id.et_add_parking_places);

        id = getIntent().getExtras().getString("id");
        address = getIntent().getExtras().getString("address");
        latitude = getIntent().getExtras().getDouble("latitude");
        longitude = getIntent().getExtras().getDouble("longitude");

        firebaseStorage = FirebaseStorage.getInstance();
    }

    public void onCancel(View view){
        finish();
    }

    public void onClickAddParking(View view){
        if(editText_places.getText().toString().isEmpty()) editText_places.setText("1");
        mParking = new Parking(id, address, "092", ("p"+id), "", false, checkBox_right_load.isChecked(), checkBox_left_load.isChecked(), checkBox_rear_load.isChecked(),
                radioButton_batery.isChecked(), radioButton_line.isChecked(), Integer.parseInt(editText_places.getText().toString()), latitude, longitude);
        Intent intent = new Intent();
        intent.putExtra("parking", mParking);
        setResult(RESULT_OK, intent);
        finish();
    }

    // Open the camera
    public void onAddPicture(View view){
        Intent takePictureIntent = new Intent(MediaStore.ACTION_IMAGE_CAPTURE);
        if (takePictureIntent.resolveActivity(getPackageManager()) != null) {
            startActivityForResult(takePictureIntent, REQUEST_IMAGE_CAPTURE);
        }
    }

    // On camera result
    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        if (requestCode == REQUEST_IMAGE_CAPTURE && resultCode == RESULT_OK) {
            Bundle bundle = data.getExtras();
            Bitmap imageBitmap = (Bitmap) bundle.get("data");

            // Resize bitmap before save it
            Bitmap resizedBitmap = Bitmap.createScaledBitmap(
                    imageBitmap, 250, 140, false);

            StorageReference storageReference = firebaseStorage.getReference()
                    .child("pictures/"+"p"+id+".jpg");
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            resizedBitmap.compress(Bitmap.CompressFormat.JPEG, 100, baos);
            byte[] datas = baos.toByteArray();

            // Upload the picture
            UploadTask uploadTask = storageReference.putBytes(datas);
            uploadTask.addOnFailureListener(new OnFailureListener() {
                @Override
                public void onFailure(@NonNull Exception exception) {
                    Toast.makeText(getBaseContext(),"Fail upload",Toast.LENGTH_LONG);
                }
            }).addOnSuccessListener(new OnSuccessListener<UploadTask.TaskSnapshot>() {
                @Override
                public void onSuccess(UploadTask.TaskSnapshot taskSnapshot) {
                    Toast.makeText(getBaseContext(),"Upload succesfull", Toast.LENGTH_LONG);

                }
            });
        }
    }
}

