package manalichricard.android.uoc.edu.discaparking;

import android.Manifest;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.graphics.BitmapFactory;
import android.location.Address;
import android.location.Geocoder;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.net.Uri;
import android.support.v4.app.ActivityCompat;
import android.support.v4.content.ContextCompat;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.view.View;
import android.widget.Button;
import android.widget.ImageButton;
import android.widget.SearchView;
import android.widget.TextView;
import android.widget.Toast;
import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.OnMapReadyCallback;
import com.google.android.gms.maps.SupportMapFragment;
import com.google.android.gms.maps.UiSettings;
import com.google.android.gms.maps.model.BitmapDescriptorFactory;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.Marker;
import com.google.android.gms.maps.model.MarkerOptions;

import java.io.IOException;
import java.util.List;
import java.util.Locale;

import manalichricard.android.uoc.edu.discaparking.manager.FirebaseManager;
import manalichricard.android.uoc.edu.discaparking.model.Parking;

public class MainActivity extends AppCompatActivity implements OnMapReadyCallback,
        LocationListener,
        GoogleMap.OnMarkerClickListener {

    //Googlemaps variables
    private GoogleMap mMap;
    private UiSettings uiSettings;
    private static final int LOCATION_REQUEST_CODE = 1;
    private LocationManager locationManager;
    private Location location;
    private double mLatitude = 0;
    private double mLongitude = 0;

    //Private variables
    private final static String POLICE_PHONE_NUMBER = "092";
    private final static int DATA_INTENT_PARKING = 1;
    private final static int DATA_INTENT_ADDPARKING = 2;
    private final static int DATA_INTENT_WARNING = 3;
    private ProgressDialog progress;
    private SearchView searchView_address;
    private Boolean isWarningSelected = false;
    private Boolean isFirebaseReady = false;
    private FirebaseManager firebaseManager;
    private Parking mParking;
    private String mParkingPosition;
    private TextView textView_latitude, textView_longitude;
    private ImageButton imageButton_location;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        searchView_address = (SearchView)findViewById(R.id.sv_search_address);
        textView_latitude = (TextView)findViewById(R.id.tv_latitude);
        textView_longitude = (TextView)findViewById(R.id.tv_longitude);
        imageButton_location = (ImageButton)findViewById(R.id.button_location);

        // If you want to show the user latlong in the toolbar
        //textView_latitude.setText(Double.toString(mLatitude));
        //textView_longitude.setText(Double.toString(mLongitude));

        instanceFirebase();

        SupportMapFragment mapFragment = (SupportMapFragment) getSupportFragmentManager()
                .findFragmentById(R.id.map);
        mapFragment.getMapAsync(this);
    }

    // Show a spinner while load Firebase database
    private void instanceFirebase() {
        progress = ProgressDialog.show(this, "Firebase", "Loading", true);
        new Thread(new Runnable() {
            @Override
            public void run()
            {
                firebaseManager = new FirebaseManager();
                while (!firebaseManager.getIsReady()){}
                isFirebaseReady = true;

                runOnUiThread(new Runnable() {
                    @Override
                    public void run()
                    {
                        imageButton_location.performClick();
                        progress.dismiss();
                    }
                });
            }
        }).start();
    }

    // Update database
    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        if (data != null) {
            if ((requestCode == DATA_INTENT_PARKING) && (resultCode == RESULT_OK)) {
                firebaseManager.closeWarning(mParkingPosition, Boolean.valueOf(data.getDataString()));
            }
            if ((requestCode == DATA_INTENT_WARNING) && (resultCode == RESULT_OK)) {
                mParking = (Parking) data.getExtras().getSerializable("parking");
                firebaseManager.addWarning(mParkingPosition, mParking);
            }
            if ((requestCode == DATA_INTENT_ADDPARKING) && (resultCode == RESULT_OK)) {
                mParking = (Parking) data.getExtras().getSerializable("parking");
                mParkingPosition = String.valueOf(firebaseManager.getParkingArrayList().size());
                firebaseManager.newParking(mParkingPosition, mParking);
            }
        }
    }

    // Search button
    public void onClickSearch(View view) {
        searchView_address.setVisibility(View.VISIBLE);
        searchView_address.setOnQueryTextListener(new SearchView.OnQueryTextListener() {
            @Override
            public boolean onQueryTextSubmit(String s) {
                String location = searchView_address.getQuery().toString();
                List<Address> addressList = null;
                getPlace(location, addressList);
                return false;
            }

            @Override
            public boolean onQueryTextChange(String s) {
                return false;
            }
        });
    }

    public void getPlace(String location, List<Address> addressList){
        if(location !=null || !location.equals("")){
            Geocoder geocoder = new Geocoder(MainActivity.this, Locale.getDefault());
            int i = 0;
            while (addressList == null && i < 3 ){
                try {
                    addressList = geocoder.getFromLocationName(location, 1);
                } catch (IOException e) {
                    e.printStackTrace();
                }
                i++;
            }
            if(addressList != null && addressList.size() > 0) {
                Address address = addressList.get(0);
                LatLng latLng = new LatLng(address.getLatitude(), address.getLongitude());
                mMap.moveCamera(CameraUpdateFactory.newLatLng(latLng));
            } else {
                Toast.makeText(getApplicationContext(), R.string.address_not_found, Toast.LENGTH_LONG).show();
            }
            //mMap.animateCamera(CameraUpdateFactory.newLatLng(latLng));
            mMap.animateCamera(CameraUpdateFactory.zoomTo(16));
            searchView_address.setVisibility(View.INVISIBLE);
        }
    }

    // Add a new parking button
    public void onClickAddParking(View view) {
        createDialog(R.string.di_add_parking, R.string.di_add_parking_inf, "OK", R.string.cancel, 1);
    }

    // Add a new warning button
    public void onClickAddWarning(View view) {
        createDialog(R.string.di_add_warning, R.string.di_add_warning_inf, "OK", R.string.cancel, 2);
    }

    // Call police button
    public void onClickCall(View view) {
        createDialog(R.string.di_call, R.string.di_call_inf, "OK", R.string.cancel, 3);
    }

    // Set the App dialogs
    public void createDialog(int title, int message, String okButton, int cancelButton, final Integer button) {
        new AlertDialog.Builder(this)
                .setTitle(title)
                .setMessage(message)
                .setPositiveButton(okButton, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialog, int which) {

                        switch (button) {
                            case 1:
                                addParking();
                                break;
                            case 2:
                                isWarningSelected = true;
                                break;
                            case 3:
                                startActivity(new Intent(Intent.ACTION_DIAL).setData(Uri.parse("tel:" + POLICE_PHONE_NUMBER)));
                                break;
                            default:
                                break;
                        }
                    }
                })
                .setNegativeButton(cancelButton, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialog, int which) {

                    }
                })
                .show();
    }

    // Intent for a new parking Activity
    private void addParking() {
        Intent intent = new Intent(MainActivity.this, AddParkingActivity.class);
        intent.putExtra("id", myId(myPosition()));
        intent.putExtra("address", getAddressFromLatLng(myPosition()));
        intent.putExtra("latitude", mLatitude);
        intent.putExtra("longitude", mLongitude);
        startActivityForResult(intent, DATA_INTENT_ADDPARKING);
    }

    // Get the user position
    public void onClickGetMyPosition(View view) {
        if (ContextCompat.checkSelfPermission(this, Manifest.permission.ACCESS_FINE_LOCATION)
                == PackageManager.PERMISSION_GRANTED) {
            mMap.setMyLocationEnabled(true); // Show user position in map

            // Get the last known user position
            locationManager = (LocationManager) getSystemService(Context.LOCATION_SERVICE);
            location = locationManager.getLastKnownLocation(LocationManager.GPS_PROVIDER);
            // Define a listener that responds to location updates
            LocationListener locationListener = new LocationListener() {
                public void onLocationChanged(Location location) {
                    // Called when a new location is found by the network location provider.
                    onNewLocation(location);
                }
                public void onStatusChanged(String provider, int status, Bundle extras) {}
                public void onProviderEnabled(String provider) {}
                public void onProviderDisabled(String provider) {}
            };
            // Register the listener with the Location Manager to receive location updates
            locationManager.requestLocationUpdates(LocationManager.NETWORK_PROVIDER  , 0, 0, locationListener);
            locationManager.requestLocationUpdates(LocationManager.GPS_PROVIDER  , 0, 0, locationListener);

            onNewLocation(location);
        } else {
            if (ActivityCompat.shouldShowRequestPermissionRationale(this,
                    Manifest.permission.ACCESS_FINE_LOCATION)) {
            } else {
                // Request permissions
                ActivityCompat.requestPermissions(
                        this,
                        new String[]{Manifest.permission.ACCESS_FINE_LOCATION},
                        LOCATION_REQUEST_CODE);
            }
        }
        setMarkersOnMap();
    }

    private void onNewLocation(Location location) {
        if(location !=null ) {
            mLatitude = location.getLatitude();
            mLongitude = location.getLongitude();
            //textView_latitude.setText(Double.toString(mLatitude));
            //textView_longitude.setText(Double.toString(mLongitude));
        }
        //setMarkersOnMap();
    }

    // Temporary positioning
    public LatLng myPosition(){
        // Change the next 3 lines by the first enabled one if you want to force a position
        //double latitude = 41.387164;
        //double longitude = 2.167955;
        //LatLng latLng = new LatLng(latitude, longitude);
        LatLng latLng = new LatLng(mLatitude, mLongitude);
        return latLng;
    }

    // Create id from Latlng
    public String myId(LatLng latLng){
        String id = latLng.latitude + "," + latLng.longitude;
        return id;
    }

    // Set the markers
    private void setMarkersOnMap() {
        int bmf=0;
        // Clean the map
        mMap.clear();

        // Iterate the list placing the markers
        for(Parking parking: firebaseManager.getParkingArrayList()) {
            bmf = parking.getIsWarning() ? R.mipmap.ic_marker_warning: R.mipmap.ic_marker;
            LatLng latLng = new LatLng(parking.getLatitude(), parking.getLongitude());
            mMap.addMarker(new MarkerOptions()
                    .position(latLng)
                    .title(getAddressFromLatLng(latLng))
                    .anchor(0.5f,1f)
                    .icon(BitmapDescriptorFactory.fromBitmap(
                            BitmapFactory.decodeResource(getResources(),
                                    bmf))));
        }

        // Set the camera to the user position
        mMap.moveCamera(CameraUpdateFactory.newLatLng(myPosition()));
        mMap.animateCamera(CameraUpdateFactory.zoomTo(16));
    }

    // Return an adress from a location
    private String getAddressFromLatLng(LatLng latLng) {
        Geocoder geocoder = new Geocoder(this, Locale.getDefault());
        String address = "";
        try {
            address = geocoder
                    .getFromLocation(latLng.latitude, latLng.longitude, 1)
                    .get(0)
                    .getAddressLine(0);
        } catch (IOException e) {
        }

        return address;
    }

    // GOOGLE MAPS METHODS ----------------------------------------------------------

    // The map is ready to be used
    @Override
    public void onMapReady(GoogleMap googleMap) {
        mMap = googleMap;

        // Google Maps controls
        uiSettings = mMap.getUiSettings();
        uiSettings.setCompassEnabled(false);
        uiSettings.setZoomControlsEnabled(false);
        uiSettings.setMyLocationButtonEnabled(false);
        uiSettings.setAllGesturesEnabled(true);

        // Listeners
        mMap.setOnMarkerClickListener(this);
    }

    @Override
    public void onLocationChanged(Location location) {
        if(location !=null ) {
            mLatitude = location.getLatitude();
            mLongitude = location.getLongitude();
            textView_latitude.setText(Double.toString(mLatitude));
            textView_longitude.setText(Double.toString(mLongitude));
        }
        //setMarkersOnMap();
    }

    @Override
    public void onStatusChanged(String s, int i, Bundle bundle) {

    }

    @Override
    public void onProviderEnabled(String s) {

    }

    @Override
    public void onProviderDisabled(String s) {

    }

    @Override
    public boolean onMarkerClick(Marker marker) {
        int position = 0;
        for (Parking parking : firebaseManager.getParkingArrayList()) {
            if (marker.getPosition().equals(new LatLng(parking.getLatitude(), parking.getLongitude()))) {
                mParking = parking;
                mParkingPosition = String.valueOf(position);
                if(isWarningSelected) {
                    isWarningSelected = false;
                    Intent intent = new Intent(MainActivity.this, WarningActivity.class);
                    intent.putExtra("parking", mParking);
                    startActivityForResult(intent, DATA_INTENT_WARNING);
                } else {
                    Intent intent = new Intent(MainActivity.this, ParkingActivity.class);
                    intent.putExtra("parking", mParking);
                    intent.putExtra("location", myPosition());
                    startActivityForResult(intent, DATA_INTENT_PARKING);
                }
            }
            position++;
        }
        return true;
    }
}
