package manalichricard.android.uoc.edu.discaparking;

import android.content.Intent;
import android.net.Uri;
import android.support.annotation.NonNull;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.view.View;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.ImageView;
import android.widget.RadioButton;
import android.widget.Switch;
import android.widget.TextView;
import android.widget.Toast;

import com.bumptech.glide.Glide;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.tasks.OnFailureListener;
import com.google.android.gms.tasks.OnSuccessListener;
import com.google.firebase.storage.FirebaseStorage;
import com.google.firebase.storage.StorageReference;

import manalichricard.android.uoc.edu.discaparking.model.Parking;

public class ParkingActivity extends AppCompatActivity implements CompoundButton.OnCheckedChangeListener {

    private Parking mParking;
    private LatLng mLocation;
    private TextView textView_address, textView_places, textView_warning;
    private Switch switch_active;
    private ImageView imageView_picture, imageView_warning;
    private CheckBox checkBox_left_load, checkBox_right_load, checkBox_rear_load;
    private RadioButton radioButton_batery, radioButton_line;
    private StorageReference mStorageReference;


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_parking);

        textView_address  = (TextView)findViewById(R.id.tv_parking_adress);
        textView_places  = (TextView)findViewById(R.id.tv_parking_places_number);
        textView_warning  = (TextView)findViewById(R.id.tv_parking_warning);
        checkBox_left_load = (CheckBox)findViewById(R.id.cb_parking_left_load);
        checkBox_right_load = (CheckBox)findViewById(R.id.cb_parking_right_load);
        checkBox_rear_load = (CheckBox)findViewById(R.id.cb_parking_rear_load);
        radioButton_batery = (RadioButton) findViewById(R.id.rb_parking_batery);
        radioButton_line = (RadioButton) findViewById(R.id.rb_parking_line);
        switch_active  = (Switch) findViewById(R.id.sw_parking_active);
        imageView_picture  = (ImageView)findViewById(R.id.iv_parking_picture);
        imageView_warning  = (ImageView)findViewById(R.id.iv_parking_warning);

        mParking = (Parking) getIntent().getExtras().getSerializable("parking");
        mLocation = (LatLng) getIntent().getExtras().getParcelable("location");

        getPicture();

        textView_address.setText(mParking.getAdress());
        textView_places.setText(String.valueOf(mParking.getPlaces()));
        checkBox_left_load.setChecked(mParking.getLeftLoad());
        checkBox_right_load.setChecked(mParking.getRightLoad());
        checkBox_rear_load.setChecked(mParking.getRearLoad());
        radioButton_batery.setChecked(mParking.getBateryParking());
        radioButton_line.setChecked(mParking.getLineParking());

        if(mParking.getIsWarning()){
            textView_warning.setVisibility(View.VISIBLE);
            textView_warning.setText(mParking.getWarning());
            switch_active.setVisibility(View.VISIBLE);
            switch_active.setOnCheckedChangeListener(this);
            imageView_warning.setVisibility(View.VISIBLE);
        }

    }

    // Get picture from Storage and publish it using Glide
    private void getPicture() {
        mStorageReference = FirebaseStorage.getInstance().getReference();
        mStorageReference.child("pictures/"+mParking.getPicture()+".jpg")
                .getDownloadUrl()
                .addOnSuccessListener(new OnSuccessListener<Uri>() {
            @Override
            public void onSuccess(Uri uri) {
                Glide.with(ParkingActivity.this)
                        .load(uri)
                        .into(imageView_picture);
            }
        }).addOnFailureListener(new OnFailureListener() {
            @Override
            public void onFailure(@NonNull Exception exception) {
                Toast.makeText(ParkingActivity.this,"No picture found",Toast.LENGTH_LONG);
            }
        });
    }

    // Warning switch control
    public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
        if (isChecked) {
            Boolean isWarning = false;
            Intent warning = new Intent();
            warning.setData(Uri.parse(String.valueOf(isWarning)));
            setResult(RESULT_OK, warning);
            finish();
        }
    }

    // Open Google Maps to get the best route
    public void onClickGoToParking(View view) {
        String uri = "http://maps.google.com/maps?saddr=" + mLocation.latitude + ","
                + mLocation.longitude + "&daddr=" + mParking.getLatitude() + ","
                + mParking.getLongitude();
        Intent intent = new Intent(Intent.ACTION_VIEW, Uri.parse(uri));
        startActivity(intent);
    }

    public void onCancel(View view){
        finish();
    }

    @Override
    public void onPointerCaptureChanged(boolean hasCapture) {

    }
}
