package manalichricard.android.uoc.edu.discaparking.manager;

import android.support.annotation.NonNull;
import android.util.Log;

import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.Task;
import com.google.firebase.auth.AuthResult;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.GenericTypeIndicator;
import com.google.firebase.database.ValueEventListener;

import java.util.ArrayList;

import manalichricard.android.uoc.edu.discaparking.model.Parking;

public class FirebaseManager {

    //Firebase variables
    private FirebaseAuth mAuth;
    private DatabaseReference mDatabase;

    //Private variables
    private Boolean isReady = false;
    private final static String TAG = "MainActivity";
    private ArrayList<Parking> parkingArrayList = new ArrayList<>();

    public FirebaseManager() {
        mAuth = FirebaseAuth.getInstance();
        mDatabase = FirebaseDatabase.getInstance().getReference();

        checkFirebaseConnection();
    }

    //Check the connection
    private void checkFirebaseConnection() {
        mDatabase = FirebaseDatabase.getInstance().getReference(".info/connected");
        mDatabase.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot snapshot) {
                boolean connected = snapshot.getValue(Boolean.class);
                if (connected) {
                    System.out.println("FIREBASE, connected");
                    mDatabase = FirebaseDatabase.getInstance().getReference("parkings");
                    checkFirebaseAuthorization();
                } else {
                    System.out.println("FIREBASE, not connected");
                }
            }

            @Override
            public void onCancelled(DatabaseError error) {
                System.err.println("Listener was cancelled");
            }
        });
    }

    // Check Firebase Authorization
    private void checkFirebaseAuthorization() {
        mAuth.signInWithEmailAndPassword("rmanalich@uoc.edu", "555555")
                .addOnCompleteListener(new OnCompleteListener<AuthResult>() {
                    @Override
                    public void onComplete(@NonNull Task<AuthResult> task) {
                        if (task.isSuccessful()) {
                            System.out.println("FIREBASE, authorized");
                            getParkingsList();
                        } else {
                            System.out.println("FIREBASE, not authorized");
                        }
                    }
                });
    }

    // Firebase database listener
    private void getParkingsList() {
        mDatabase = FirebaseDatabase.getInstance().getReference("parkings");
        // Read from the database
        mDatabase.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                getParkingsFromDataSnapshot(dataSnapshot);
            }

            @Override
            public void onCancelled(DatabaseError error) {
                // Failed to read value
                Log.w(TAG, "Failed to read value.", error.toException());
            }
        });
    }

    // Get the parking list from database
    private void getParkingsFromDataSnapshot(DataSnapshot dataSnapshot) {
        isReady = false;
        GenericTypeIndicator<ArrayList<Parking>> genericTypeIndicator =
                new GenericTypeIndicator<ArrayList<Parking>>() {};
        final ArrayList<Parking> values = dataSnapshot.getValue(genericTypeIndicator);

        parkingArrayList = values;
        isReady = true;
    }

    public Boolean getIsReady(){
        return isReady;
    }

    public ArrayList<Parking> getParkingArrayList() {
        return parkingArrayList;
    }

    // Add a parking warning
    public void addWarning(String position, Parking parking) {
        mDatabase = FirebaseDatabase.getInstance().getReference();
        mDatabase.child("parkings").child(position).child("isWarning").setValue(parking.getIsWarning());
        mDatabase.child("parkings").child(position).child("warning").setValue(parking.getWarning());
    }

    // Remove a parking warning
    public void closeWarning(String position, Boolean isWarning) {
        mDatabase = FirebaseDatabase.getInstance().getReference();
        mDatabase.child("parkings").child(position).child("isWarning").setValue(isWarning);
    }

    // Add a new parking
    public void newParking(String position, Parking parking) {
        mDatabase = FirebaseDatabase.getInstance().getReference();
        mDatabase.child("parkings").child(position).setValue(parking);
    }
}
